C     Last change:  HRO  10 Jun 2005    4:21 pm


C
C                     ******************
C                      Read_ini_routines
C                     ******************
C

C  DESCRIPTION:
C -------------
C
C     Set of subroutines to read an ni file.
C     Base on a parser written by Douglas S. Elder, elderdo@yahoo.com
C     Comments and additional routines supplied by H.R. Olesen.
C     Found on www.fortranlib.com, 
C     
C     The three most important routines from a user point of view are:
C     
C     setIniFilename(value, is_ini_found)
C        The default ini File is Options.ini, but this routine allows 
C        you to use a different iniFile or to switch to another.
C        Further, ist checks whether the file exists.
C             
C     
C     getValue(section, kwd, value, default, is_found)
C        Search for the Section and keyword. If found return its value,
C        otherwise return the default
C
C     getValueOnOff(section, kwd, DEFAULT, is_found)
C        Boolean function. Search for the Section and Keyword.
C        The keyword should be either 'On' or 'Off'.
C        The function value is True or False. Default indicates the default value.
C     
C     Other routines in the package:
C     
C     SUBROUTINE loadOptions
C     Read in the iniFile into the common Options block
C     
C     SUBROUTINE getValueX(section, kwd, value)
C     Try to find the Section and keyword, if found return its value
C     otherwise return an empty string
C     
C     INTEGER FUNCTION firstChar (string)
C     Return the index of the first non-blank
C     character in a string
C     
C     INTEGER FUNCTION lastChar(string)
C     Return the index of the last non-blank
C     character in a string
C     
C  
C  PROGRAM USAGE:
C  --------------
C
C     setIniFilename(value, is_found) can be used to select an ini file
C     and check whether it exists.
C     value contains the name of the file, and is_found (a logical) reports
C     whether the file is found.
C
C     getValue(section, kwd, value, default, is_found)
C     There are two output parameters: value and is_found
C     is_found is a logical, all other parameters are strings.
C
C    *****************  End of program header  ***************




C    ************************************************************
C     Common block
C    ************************************************************
C** common block that will contain the iniFile
      BLOCK DATA iniFile
      LOGICAL initialized
      INTEGER CNT
      CHARACTER LINES(100)*256
      CHARACTER iniFilename*256
      COMMON /Options/ CNT, LINES, initialized, iniFilename
      DATA initialized, CNT, iniFilename / .FALSE., 0, 'Options.ini' /
      END


C    ************************************************************
      SUBROUTINE setIniFilename(value, is_ini_found)
C    ************************************************************
C** Allows you to use a different iniFile or to switch to another.
C     Also checks whether the file exists.

      CHARACTER value*(*)
      LOGICAL initialized, is_ini_found
      INTEGER CNT
      CHARACTER LINES(100)*256
      CHARACTER iniFilename*256
      COMMON /Options/ CNT, LINES, initialized, iniFilename
      if (initialized .EQV. .TRUE.) then
        if (value .NE. iniFilename) then
            iniFilename = value
C****** switching to a different ini file
                call loadOptions

        end if
      else
C****** overriding the default ini file
        iniFilename = value
      end if
      
      is_ini_found=.TRUE.
      OPEN(UNIT=33, FILE=iniFilename, STATUS='OLD', ERR=5)
      GOTO 10
      
5     is_ini_found=.FALSE.
      
10    CONTINUE      
      END


C    ************************************************************
      SUBROUTINE getValue(section, kwd, value, DEFAULT, is_found)
C    ************************************************************
C***** search for the Section and keyword, if found return its value
C***** otherwise return the default
      
      IMPLICIT NONE
      CHARACTER section*(*)
      CHARACTER kwd*(*)
      CHARACTER value*(*)
      CHARACTER default*(*)
      INTEGER I
      LOGICAL is_found
C**      WRITE(*,*) 'getValue', section, kwd, value, default
      value = ''
      call getValueX(section, kwd, value)
C**      WRITE(*,*) 'Got value ', value
      is_found=.TRUE.
      if (value .EQ. '') then
         is_found=.FALSE.
         value = default
      end if
      END

C    ************************************************************
      Logical FUNCTION getValueOnOff(section, kwd, DEFAULT, is_found)
C    ************************************************************
C***** Search for the Section and keyword.
C      We presume that the keyword is On or Off, and return
C      accordingly True or False as the value of the function.
C***** otherwise return the default
      
      IMPLICIT NONE
      CHARACTER section*(*)
      CHARACTER kwd*(*)
      CHARACTER value*80
      Logical Default
      INTEGER I
      LOGICAL is_found
C      WRITE(*,*) 'getValue ', section, kwd, default
      value = ''
      call getValueX(section, kwd, value)
C      WRITE(*,*) 'Got value ', value
      is_found=.TRUE.

      CALL Upcase(value)
      If (value .EQ. 'ON') Then
         getValueOnOff=.True.
      Else If (value .EQ. 'OFF') Then
         getValueOnOff=.False.
      Else
         is_found=.FALSE.
         getValueOnOff = default
      end if
      END






C    ************************************************************
      SUBROUTINE loadOptions
C    ************************************************************      
C**** read in the iniFile into the common Options block
      
      IMPLICIT NONE     
      INTEGER CNT
      LOGICAL initialized
      CHARACTER LINES(100)*256
      CHARACTER iniFilename*256
      COMMON /Options/ CNT, LINES, initialized, iniFilename
      CHARACTER LINE*256 
C**      WRITE(*,*) 'loadOptions'
      CNT = 0
      OPEN(UNIT=33, FILE=iniFilename)
1     READ(33,'(A)', END=10) LINE
      CNT = CNT + 1
      if (CNT .GT. 100) then
C**         WRITE(0,*) 'Options.ini file > 100 lines.'
      STOP 16
      else
         LINES(CNT) = LINE 
      end if
      GOTO 1
10    CONTINUE
C**      WRITE(*,*) 'CNT = ',CNT
      CLOSE (UNIT=33)
      initialized = .TRUE.
      END



C    ************************************************************
      SUBROUTINE getValueX(section, kwd, value)
C    ************************************************************
C*** try to find the Section and keyword, if found return its value
C*** otherwise return an empty string

      IMPLICIT NONE
      CHARACTER section*(*)
      CHARACTER kwd*(*)
      CHARACTER value*(*)
      INTEGER I, J, STARTVAL
      INTEGER CNT
      LOGICAL initialized, is_ini_found
      CHARACTER LINES(100)*256
      CHARACTER iniFilename*256
      COMMON /Options/ CNT, LINES, initialized, iniFilename
      INTEGER MAXLINE
      PARAMETER (MAXLINE = 256)
      CHARACTER LINE*256
      LOGICAL foundSection, foundKwd
      if (initialized .EQV. .FALSE.) then
            call loadOptions
      end if
      foundSection = .FALSE.
      foundKwd = .FALSE.
      value = ''
C**      WRITE(*,*) 'Looking for ',section
      DO I=1, CNT
         if (LINES(I)(1:1) .EQ. '[') then
      DO J=2, MAXLINE
        if (LINES(I)(J:J) .EQ. ']') then
         if (section .EQ. LINES(I)(2:J-1)) then
            foundSection = .TRUE.
C**            Write(*,*) 'foundSection=', foundSection
            GOTO 15
         end if
        end if
      ENDDO
      else
      if (foundSection .EQV. .TRUE.) then
                        STARTVAL = 0
C**         WRITE(*,*) 'Looking for keyword ', kwd
         DO J=1, MAXLINE
            if (LINES(I)(J:J) .EQ. '=') then
               if (kwd .EQ. LINES(I)(1:J-1)) then
                  STARTVAL = J+1
                  foundKwd = .TRUE.
C**                  Write(*,*) 'found keyword ', kwd
                  GOTO 13
               end if
            end if
         END DO
13       CONTINUE
         if (foundKwd .EQV. .TRUE.) then
            DO J=MAXLINE, STARTVAL, -1
C**               Write(*,*) 'Looking for value at ', STARTVAL, J
               if (LINES(I)(J:J) .NE. '') then
                  value = LINES(I)(STARTVAL:J)
C**                  Write(*,*) 'Got value at line ', I, ' pos ', STARTVAL, J
                  GOTO 20
               end if
            END DO
         end if
      end if
      end if
15    CONTINUE
      ENDDO
20    CONTINUE 
      END


C    ************************************************************
C  Two string routines follow (used by the above routines)
C  Ver 1.0     
C  Date: 23 June 03
C  Written by: Douglas S. Elder elderdo@yahoo.com
C    ************************************************************

C    ************************************************************
      INTEGER FUNCTION firstChar (string)
C    ************************************************************
C  return the index of the first non-blank
C  character in a string

      IMPLICIT NONE
      CHARACTER string*(*)
      INTEGER I

      DO I=1,LEN(string)
         IF (string(I:I) .NE. ' ') THEN
            firstChar = I
            RETURN
         END IF
      END DO
      END

C    ************************************************************
      INTEGER FUNCTION lastChar(string)
C    ************************************************************
C  return the index of the last non-blank
C  character in a string

      IMPLICIT NONE
      CHARACTER string*(*)
      INTEGER I
      DO I=LEN(string), 1, -1
         IF (string(I:I) .NE. ' ') THEN
            lastChar = I
            RETURN
         END IF
      END DO
      END



C    ************************************************************
      Subroutine UPCASE (STRING)
C    ************************************************************
C     Subroutines UPCASE and LOWCASE converts the case of strings.
C     Use only characters A-Z, not international characters.

      implicit none
      character (len=*),INTENT(INOUT) :: STRING
      integer I
      character(len=1) C

      do I = 1,LEN(STRING)
        C = STRING(I:I)
        if ((C >= 'a') .and. (C <= 'z')) then
         STRING(I:I) = CHAR(ICHAR(C)-32)
        end if
      end do
      return
      end subroutine UPCASE
      

C    ************************************************************
      Subroutine DOWNCASE (STRING)
C    ************************************************************      
C     Subroutines UPCASE and LOWCASE converts the case of strings.
C     Use only characters A-Z, not international characters.    

      implicit none
      character (len=*),INTENT(INOUT) :: STRING
      integer I
      character(len=1) C

      do I = 1,LEN(STRING)
        C = STRING(I:I)
        if ((C >= 'A') .and. (C <= 'Z')) then
          STRING(I:I) = CHAR(ICHAR(C)+32)
        end if
        end do
      return
      end subroutine DOWNCASE
